var component = require( "./lib/component" );
var metrics = require( "./lib/metrics" );
var request = require( "request" );

var nginx = component.factory({
	guid: "com.nginx.newrelic-agent",
	duration: 60,
	agent: { name: "node-newrelic-nginx", version: "0.0.0" },
	metrics: {
		accept: new metrics.Counter( "Connections/Accepted", "Connections/sec" ),
		dropped: new metrics.Counter( "Connections/Dropped", "Connections/sec" ),
		idle: [
			new metrics.Metric( "Connections/Idle", "Connections" ),
			new metrics.Metric( "ConnSummary/Idle", "Connections" )
		],
		active: [
			new metrics.Metric( "Connections/Active", "Connections") ,
			new metrics.Metric( "ConnSummary/Active", "Connections" )
		],
		total: new metrics.Counter( "Requests/Total", "Requests/sec" ),
		current: new metrics.Metric( "Requests/Current", "Requests" )
	},
	getStats: function( callback ) {
		request( this.nginxUri, function( err, response ) {
			var re = /Active connections: (\d+) \n.+\n (\d+) (\d+) (\d+) \nReading: (\d+) Writing: (\d+) Waiting: (\d+)/;
			var match = re.exec( response.body );
			if ( match ) {
				callback({
					active: match[ 1 ],
					accept: match[ 2 ],
					handled: match[ 3 ],
					total: match[ 4 ],
					reading: match[ 5 ],
					writing: match[ 6 ],
					idle: match[ 7 ]
				});
			} else {
				console.error( "not what i expected", response.body );
			}
		});
	},
	poll: function( callback )  {
		this.getStats( function( data ) {
			this.read( data );
			callback();
		}.bind( this ) );
	},
	init: function( callback ) {
		this.getStats( function( data ) {
			this.seed( data );
			callback();
		}.bind( this ) );
	}
});

new nginx({
	apiKey: "db51e858874428d54a2f6e98b7a81359131e5ca8",
	name: "jq01.jquery.com",
	nginxUri: "http://localhost:31337/nginx_status"
}).start();
